#!/usr/bin/env python3
# Copyright (C) 2019 Checkmk GmbH - License: GNU General Public License v2
# This file is part of Checkmk (https://checkmk.com). It is subject to the terms and
# conditions defined in the file COPYING, which is part of this source code package.

# enhancements by thl-cmk[at]outlook[dot]com, https://thl-cmk.hopto.org
# - changed remaining time (WATO) from Age (Days, Hours, Minutes, Seconds) to Days only
# - added WATO option for License state is not ok -> default to WARN
# - added discovery rule for ITEM variant (Org Name/Org ID - this is the default, Org Name, Org ID)

# 2023-11-18: moved discovery rule set to cisco_meraki_organisations for reuse with cisco_meraki_organisations_api
# 2024-06-28: refactored for CMK 2.3
# 2024-06-30: renamed from cisco_meraki_org_licenses_overviewi.py in to licenses_overviewi.py

from cmk.rulesets.v1 import Label, Title, Help
from cmk.rulesets.v1.form_specs import (
    DefaultValue,
    DictElement,
    Dictionary,
    FixedValue,
    ServiceState,
    SimpleLevels,
    LevelDirection,
    TimeSpan,
    TimeMagnitude,
    validators,
)
from cmk.rulesets.v1.rule_specs import CheckParameters, HostAndItemCondition, Topic

_DAY = 60.0 * 60.0 * 24.0


def _migrate(p: str | dict) -> dict[str, any]:
    # change age to days, we suspect there is no warning less than 1 hour in seconds,
    # and we will not warn/crit above 3600 days (10 years)
    if p.get('remaining_expiration_time'):
        warn, crit = p.get('remaining_expiration_time')
        if warn >= 3600:
            warn = int(warn / 86400)
        if crit >= 3600:
            crit = int(crit / 86400)
        p['remaining_expiration_time'] = (warn, crit)
    return p


def _parameter_form() -> Dictionary:
    return Dictionary(
        elements={
            "remaining_expiration_time": DictElement(
                parameter_form=SimpleLevels[float](
                    title=Title("Remaining licenses expiration time"),
                    help_text=Help(""),
                    form_spec_template=TimeSpan(
                        displayed_magnitudes=[TimeMagnitude.DAY],
                        custom_validate=(validators.NumberInRange(min_value=0),),
                    ),
                    level_direction=LevelDirection.LOWER,
                    prefill_fixed_levels=DefaultValue((40 * _DAY, 20 * _DAY)),
                )),
            # SimpleLevels[int](
            #     title=Title("Lower levels for remaining expiration time of licenses"),
            #     level_direction=LevelDirection.LOWER,
            #     # elements=[
            #         #     Integer(title=_("Warning at"), unit='Days'),
            #         #     Integer(title=_("Critical at"), unit='Days'),
            #         # ],
            #     )),
            "state_license_not_ok": DictElement(
                parameter_form=ServiceState(
                    title=Title('Monitoring state if License state is not OK'),
                    prefill=DefaultValue(ServiceState.WARN),
                )),
            'dont_show_alias_on_info': DictElement(
                parameter_form=FixedValue(
                    value=True,
                    title=Title('Don\'t show alias on info line'),
                    label=Label(''),
                    # help_test=Help(
                    #     'The alias is the Organisation ID or the Organisation name, depending on the Item.\n'
                    #     'If the item is the Organisation ID, the alias is the Organisation name and vice versa.\n'
                    #     'Organisation ID and Organisation name will always shown in the service details.'
                    # )
                ))
        },
        # migrate=_migrate,
        # ignored_keys=[
        #     'internal_item_name',
        #     'old_item_name',
        #     'item_variant',
        # ],
    )


rule_spec_cisco_meraki_org_licenses_overview = CheckParameters(
    name="cisco_meraki_org_licenses_overview",
    topic=Topic.NETWORKING,
    parameter_form=_parameter_form,
    title=Title("Cisco Meraki Organisation Licenses Overview"),
    condition=HostAndItemCondition(item_title=Title('Organization')),
)
